package tk.antoine.roux.architecture;

import com.tngtech.archunit.core.domain.JavaClasses;
import com.tngtech.archunit.core.importer.ImportOption.DoNotIncludeArchives;
import com.tngtech.archunit.core.importer.ImportOption.DoNotIncludeJars;
import com.tngtech.archunit.core.importer.ImportOption.DoNotIncludePackageInfos;
import com.tngtech.archunit.junit.AnalyzeClasses;
import com.tngtech.archunit.junit.ArchTest;
import com.tngtech.archunit.library.Architectures;

/**
 * Test of clean architecture package dependencies
 */
@AnalyzeClasses(packages = "tk.antoine.roux", importOptions = {
    DoNotIncludeArchives.class,
    DoNotIncludeJars.class,
    DoNotIncludePackageInfos.class
})
class CleanArchitectureTest {

    @ArchTest
    void should_respect_clean_architecture_application_layer(JavaClasses classes) {
        Architectures.layeredArchitecture()
            .consideringAllDependencies()

            //@formatter:off
            .layer(Layer.DOMAIN.name).definedBy(Layer.DOMAIN.path)
                .layer(Layer.MODEL.name).definedBy(Layer.MODEL.path)
                .optionalLayer(Layer.REPOSITORIES.name).definedBy(Layer.REPOSITORIES.path)
                .optionalLayer(Layer.USECASES.name).definedBy(Layer.USECASES.path)
            .layer(Layer.INFRASTRUCTURE.name).definedBy(Layer.INFRASTRUCTURE.path)
                .optionalLayer(Layer.ADAPTER_IN.name).definedBy(Layer.ADAPTER_IN.path)
                .optionalLayer(Layer.ADAPTER_OUT.name).definedBy(Layer.ADAPTER_OUT.path)
            //@formatter:on

            .whereLayer(Layer.INFRASTRUCTURE.name).mayNotBeAccessedByAnyLayer()
            .whereLayer(Layer.USECASES.name).mayOnlyBeAccessedByLayers(Layer.ADAPTER_IN.name)
            .whereLayer(Layer.REPOSITORIES.name).mayOnlyBeAccessedByLayers(Layer.USECASES.name)
            .whereLayer(Layer.MODEL.name).mayOnlyBeAccessedByLayers(Layer.INFRASTRUCTURE.name)
            .whereLayer(Layer.ADAPTER_OUT.name).mayNotBeAccessedByAnyLayer()
            .whereLayer(Layer.ADAPTER_OUT.name).mayNotAccessAnyLayer()

            .check(classes);
    }

}
